/*!   
    \file tool.cpp
    \brief Tool class
    \author zafaco GmbH <info@zafaco.de>
    \date Last update: 2017-07-19

    Copyright (C) 2016 - 2017 zafaco GmbH

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3 
    as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "tool.h"


CTool::CTool()
{
}

CTool::~CTool()
{
}

string CTool::get_timestamp_string()
{
    string date;

    time_t now = time(0);

    char* dt = ctime(&now);

    date = string( dt );

    date.erase( remove(date.begin(), date.end(), '\r'), date.end() );
    date.erase( remove(date.begin(), date.end(), '\n'), date.end() );

    return date;
}

void CTool::toLower(string &sText)
{
    for(unsigned int n = 0; n < sText.size(); n++)
    {
        sText.at(n) = tolower(sText.at(n));
    }
}

string CTool::get_ip_str(const struct sockaddr *sa)
{
    string ip;
    switch(sa->sa_family)
    {
        case AF_INET:
            char s1[INET_ADDRSTRLEN];
            inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr),s1, INET_ADDRSTRLEN);
            ip = string(s1);
            break;

        case AF_INET6:
            char s2[INET6_ADDRSTRLEN];
            inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr),s2, INET6_ADDRSTRLEN);
            ip = string(s2);
            break;

        default:
            return "0.0.0.0";
    }

    return ip;
}

string CTool::getHostname()
{
    char hostname[1024];
    gethostname(hostname, 1024);
    return string(hostname);
}

void CTool::replaceStringInPlace(string& subject, const string& search, const string& replace) 
{
    size_t pos = 0;
    while((pos = subject.find(search, pos)) != string::npos) 
    {
        subject.replace(pos, search.length(), replace);
        pos += replace.length();
    }
}

void CTool::tokenize(string &str,vector<string> &tokens, string &delimiters)
{
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while( string::npos != pos || string::npos != lastPos )
    {
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        lastPos = str.find_first_not_of(delimiters, pos);
        pos = str.find_first_of(delimiters, lastPos);
    }
}

unsigned long long CTool::get_timestamp()
{
    struct timeval tp;

    gettimeofday(&tp,NULL);

    return ( ( (unsigned long long)(tp.tv_sec) * 1000000 ) + (unsigned long long)(tp.tv_usec));
}

int CTool::randomData()
{
    return (255 * (rand() / (RAND_MAX + 1.0)) );
}

int CTool::randomData(vector<char> &vVector, int size)
{	
    srand(CTool::get_timestamp());

    for( int i = 0; i < size; i++ )
        vVector.push_back( (char)randomData() );

    return 0;
}

int CTool::randomData(char *sbuffer, int size)
{		
    srand(CTool::get_timestamp());

    for (int counter = 0; counter < size; counter++)
    {
        sbuffer[counter] = (char)randomData();
    }

    return 0;
}

int CTool::get_timestamp_usec()
{
    struct timeval tp;

    gettimeofday(&tp,NULL);

    return tp.tv_usec;
}

int CTool::get_timestamp_sec()
{
    struct timeval tp;

    gettimeofday(&tp,NULL);

    return tp.tv_sec;
}