/*!   
    \file wslistener.cpp
    \brief WebSocket listener
    \author zafaco GmbH <info@zafaco.de>
    \date Last update: 2017-07-19

    Copyright (C) 2016 - 2017 zafaco GmbH

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3 
    as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "wslistener.h"


using namespace std;


CWsListener::CWsListener()
{
}

CWsListener::~CWsListener()
{
    delete(mSocket);
}

CWsListener::CWsListener(int nPort)
{
    mPort       = nPort;
    
    mSocket     = new CConnection();
}


int CWsListener::run()
{
    signal(SIGCLD, SIG_IGN);

    int on = 1;

    struct sockaddr_in6 client;
    unsigned int clientlen = sizeof (client);

    if ((mSock = mSocket->tcp6SocketServer(mPort)) == 1)
    {
        TRC_CRIT("Socket creation failed - Could not establish connection on Port: " + to_string(mPort));
        return EXIT_FAILURE;
    }
    
    TRC_INFO("Start Thread: WS Listener on Port: " + to_string(mPort) + " with PID: " + std::to_string(syscall(SYS_gettid)));

    timeval tv;
    tv.tv_sec = 60;
    tv.tv_usec = 0;

    while (::RUNNING)
    {
        int nSocket = accept(mSock, (struct sockaddr *) &client, &clientlen);
        
        setsockopt(nSocket, SOL_SOCKET, SO_RCVTIMEO, (timeval *) &tv, sizeof (timeval));
        setsockopt(nSocket, SOL_SOCKET, SO_SNDTIMEO, (timeval *) &tv, sizeof (timeval));
        
        setsockopt(nSocket, IPPROTO_TCP, TCP_QUICKACK,  (void *) &on, sizeof (on));

        if (fork() == 0)
        {
            string ip = CTool::get_ip_str((struct sockaddr *)&client);  
            
            if (ip.find("::ffff:") != string::npos)
            {
                ip = ip.substr(7,string::npos);
            }
            
            TRC_DEBUG("---------------------------------------------------------");
            TRC_INFO("Socket: wsListener: Connection Received from Client IP: " + ip);
            
            CWsHandler *pWsHandler = new CWsHandler(&client, nSocket, ip);

            pWsHandler->handle_websocket();

            TRC_INFO("Socket: wsListener: Connection Shutdown for Client IP: " + ip);
            
            close(nSocket);
            
            exit(0);
        }

        close(nSocket);
    }
    
    close(mSock);

    TRC_DEBUG("End Thread: WS Listener with PID: " + std::to_string(syscall(SYS_gettid)));

    return 0;
}